BEGIN;

-- 1.0pre5 -> jmd-1.0pre6
-- Stellenanteile sind nun vom typ numeric (mpuls/issue851)

SET ROLE :db_adm;

INSERT INTO db_history (beschreibung, zeitpunkt)
VALUES ('jmd-1.0pre6/01-stellenanteile.sql', current_timestamp);

DROP VIEW master_tbl_eval_total_view CASCADE;
DROP VIEW master_tbl_view CASCADE;

ALTER TABLE master_tbl ALTER anzahl_ha_stellenteile TYPE NUMERIC; 
ALTER TABLE master_tbl ALTER anzahl_nha_stellenteile TYPE NUMERIC; 

-- master_tbl_view
CREATE OR REPLACE VIEW master_tbl_view
AS SELECT master_tbl.* FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin','MEMBER');

CREATE OR REPLACE RULE master_tbl_update_rule AS ON UPDATE TO master_tbl_view
DO INSTEAD UPDATE master_tbl SET
    phase = NEW.phase,
    datum_begin_zeitraum = NEW.datum_begin_zeitraum,
    datum_ende_zeitraum = NEW.datum_ende_zeitraum,
    raeumliche_zustaendigkeit = NEW.raeumliche_zustaendigkeit,
    nennung_zustaendigkeit = NEW.nennung_zustaendigkeit,
    jmdstruktur = NEW.jmdstruktur,
    anzahl_ha_stellenteile = NEW.anzahl_ha_stellenteile,
    anzahl_ha_personen = NEW.anzahl_ha_personen,
    anzahl_ha_maennlich = NEW.anzahl_ha_maennlich,
    anzahl_ha_weiblich = NEW.anzahl_ha_weiblich,
    anzahl_ha_migration = NEW.anzahl_ha_migration,
    ha_sprachkenntnis_arabisch = NEW.ha_sprachkenntnis_arabisch,
    ha_sprachkenntnis_englisch = NEW.ha_sprachkenntnis_englisch,
    ha_sprachkenntnis_franzoesisch = NEW.ha_sprachkenntnis_franzoesisch,
    ha_sprachkenntnis_polnisch = NEW.ha_sprachkenntnis_polnisch,
    ha_sprachkenntnis_russisch = NEW.ha_sprachkenntnis_russisch,
    ha_sprachkenntnis_spanisch = NEW.ha_sprachkenntnis_spanisch,
    ha_sprachkenntnis_tuerkisch = NEW.ha_sprachkenntnis_tuerkisch,
    ha_sprachkenntnis_sonstige = NEW.ha_sprachkenntnis_sonstige,
    ha_sprachkenntnisse_sonstige_nennung = NEW.ha_sprachkenntnisse_sonstige_nennung,
    verteilung_aa_begleitung_cm = NEW.verteilung_aa_begleitung_cm,
    verteilung_aa_betreuung = NEW.verteilung_aa_betreuung,
    verteilung_aa_gruppenangebote = NEW.verteilung_aa_gruppenangebote,
    verteilung_aa_netzwerkarbeit = NEW.verteilung_aa_netzwerkarbeit,
    verteilung_aa_oeffentlichkeit = NEW.verteilung_aa_oeffentlichkeit,
    verteilung_aa_evaluation = NEW.verteilung_aa_evaluation,
    verteilung_aa_dokumentation = NEW.verteilung_aa_dokumentation,
    verteilung_aa_sonstiges = NEW.verteilung_aa_sonstiges,
    verteilung_aa_sonstiges_nennung = NEW.verteilung_aa_sonstiges_nennung,
    anzahl_nha_stellenteile = NEW.anzahl_nha_stellenteile,
    anzahl_nha_personen = NEW.anzahl_nha_personen,
    anzahl_nha_maennlich = NEW.anzahl_nha_maennlich,
    anzahl_nha_weiblich = NEW.anzahl_nha_weiblich,
    anzahl_nha_migration = NEW.anzahl_nha_migration,
    anzahl_nha_stundenunfang = NEW.anzahl_nha_stundenunfang,
    anzahl_ea_personen = NEW.anzahl_ea_personen,
    anzahl_ea_maennlich = NEW.anzahl_ea_maennlich,
    anzahl_ea_weiblich = NEW.anzahl_ea_weiblich,
    anzahl_ea_migration = NEW.anzahl_ea_migration,
    anzahl_gesamt_personen = NEW.anzahl_gesamt_personen,
    nennung_qualimanagement = NEW.nennung_qualimanagement,
    zielgruppen_netzwerk = NEW.zielgruppen_netzwerk,
    zielgruppen_netzwerk_nennung = NEW.zielgruppen_netzwerk_nennung,
    mitarbeit_netzwerk = NEW.mitarbeit_netzwerk,
    mitarbeit_netzwerk_erlaueterung = NEW.mitarbeit_netzwerk_erlaueterung,
    erstellung_netzwerkkarte = NEW.erstellung_netzwerkkarte,
    beteiligung_integrationskonzepte = NEW.beteiligung_integrationskonzepte,
    beteiligung_integrationskonzepte_erlaeuterung = NEW.beteiligung_integrationskonzepte_erlaeuterung,
    teilnahme_jugenhilfekonferenz = NEW.teilnahme_jugenhilfekonferenz,
    teilnahme_jugenkonferenz = NEW.teilnahme_jugenkonferenz,
    mitarbeit_gremien = NEW.mitarbeit_gremien,
    weiter_akt_art_interkulturell = NEW.weiter_akt_art_interkulturell,
    weiter_akt_art_aufsuchende = NEW.weiter_akt_art_aufsuchende,
    weiter_akt_art_eltern = NEW.weiter_akt_art_eltern,
    weiter_akt_art_freiwilligerdienst = NEW.weiter_akt_art_freiwilligerdienst,
    weiter_akt_art_sonstiges = NEW.weiter_akt_art_sonstiges,
    weitere_akt_anzahl_informationsveranst = NEW.weitere_akt_anzahl_informationsveranst,
    weitere_akt_anzahl_workshops = NEW.weitere_akt_anzahl_workshops,
    weitere_akt_anzahl_trainings = NEW.weitere_akt_anzahl_trainings,
    weitere_akt_anzahl_veroeffentlichungen = NEW.weitere_akt_anzahl_veroeffentlichungen,
    weitere_akt_anzahl_sonstiges = NEW.weitere_akt_anzahl_sonstiges
WHERE id = NEW.id;

CREATE OR REPLACE RULE master_tbl_delete_rule AS ON DELETE TO master_tbl_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE master_tbl_insert_rule AS ON INSERT TO master_tbl_view
    DO INSTEAD NOTHING;

GRANT SELECT ON master_tbl_view TO GROUP :cm_group;
GRANT UPDATE ON master_tbl_view TO GROUP :cm_group;
GRANT SELECT ON master_tbl_view TO GROUP :admin_group;

-- master_tbl_anonym_view
CREATE OR REPLACE VIEW master_tbl_anonym_view
AS SELECT
    id,
    uuid_id,
    bearbeiter_id,
    phase,
    datum_begin_zeitraum,
    datum_ende_zeitraum,
    raeumliche_zustaendigkeit,
    nennung_zustaendigkeit,
    jmdstruktur,
    anzahl_ha_stellenteile,
    anzahl_ha_personen,
    anzahl_ha_maennlich,
    anzahl_ha_weiblich,
    anzahl_ha_migration,
    ha_sprachkenntnis_arabisch,
    ha_sprachkenntnis_englisch,
    ha_sprachkenntnis_franzoesisch,
    ha_sprachkenntnis_polnisch,
    ha_sprachkenntnis_russisch,
    ha_sprachkenntnis_spanisch,
    ha_sprachkenntnis_tuerkisch,
    ha_sprachkenntnis_sonstige,
    ha_sprachkenntnisse_sonstige_nennung,
    verteilung_aa_begleitung_cm,
    verteilung_aa_betreuung,
    verteilung_aa_gruppenangebote,
    verteilung_aa_netzwerkarbeit,
    verteilung_aa_oeffentlichkeit,
    verteilung_aa_evaluation,
    verteilung_aa_dokumentation,
    verteilung_aa_sonstiges,
    verteilung_aa_sonstiges_nennung,
    anzahl_nha_stellenteile,
    anzahl_nha_personen,
    anzahl_nha_maennlich,
    anzahl_nha_weiblich,
    anzahl_nha_migration,
    anzahl_nha_stundenunfang,
    anzahl_ea_personen,
    anzahl_ea_maennlich,
    anzahl_ea_weiblich,
    anzahl_ea_migration,
    anzahl_gesamt_personen,
    nennung_qualimanagement,
    zielgruppen_netzwerk,
    zielgruppen_netzwerk_nennung,
    mitarbeit_netzwerk,
    mitarbeit_netzwerk_erlaueterung,
    erstellung_netzwerkkarte,
    beteiligung_integrationskonzepte,
    beteiligung_integrationskonzepte_erlaeuterung,
    teilnahme_jugenhilfekonferenz,
    teilnahme_jugenkonferenz,
    mitarbeit_gremien,
    weiter_akt_art_interkulturell,
    weiter_akt_art_aufsuchende,
    weiter_akt_art_eltern,
    weiter_akt_art_freiwilligerdienst,
    weiter_akt_art_sonstiges,
    weitere_akt_anzahl_informationsveranst,
    weitere_akt_anzahl_workshops,
    weitere_akt_anzahl_trainings,
    weitere_akt_anzahl_veroeffentlichungen,
    weitere_akt_anzahl_sonstiges FROM master_tbl_view;

CREATE OR REPLACE RULE master_tbl_anonym_delete_rule AS ON DELETE TO master_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE master_tbl_anonym_insert_rule AS ON INSERT TO master_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON master_tbl_anonym_view TO GROUP :cm_group;
GRANT SELECT ON master_tbl_anonym_view TO GROUP :admin_group;

-- master_tbl_eval_total_view
CREATE OR REPLACE  VIEW master_tbl_eval_total_view AS
    SELECT m.* FROM master_tbl m JOIN ka_status_tbl s
        ON m.id = s.master_id
        WHERE
            s.status <> 3 AND
            (m.bearbeiter_id = (getuserid())::integer)
            OR pg_has_role('admin','MEMBER');

GRANT SELECT ON master_tbl_eval_total_view TO :cm_group;
GRANT SELECT ON master_tbl_eval_total_view TO :admin_group;

-- master_tbl_eval_incm_view
CREATE OR REPLACE  VIEW master_tbl_eval_incm_view AS
    SELECT * FROM master_tbl_eval_total_view
        WHERE phase IN (1);

GRANT SELECT ON public.master_tbl_eval_incm_view TO :cm_group;
GRANT SELECT ON public.master_tbl_eval_incm_view TO :admin_group;

CREATE OR REPLACE  VIEW master_tbl_incm_view AS
    SELECT * FROM master_tbl_view
        WHERE phase IN (1);

GRANT SELECT ON public.master_tbl_incm_view TO :cm_group;
GRANT SELECT ON public.master_tbl_incm_view TO :admin_group;

-- Finish
SET ROLE postgres;

COMMIT;
