-- Update compute_case_structure to make it identical in both upgraded
-- and newly installed databases. Only the order in which the repeat
-- groups are mentioned changes, but this change would show up in the
-- output of dumpnormalizedschema.sh as a false positive.

-- PLPython is an untrusted language. -> Need to be postgres.
SET ROLE postgres;

CREATE OR REPLACE FUNCTION compute_case_structure(case_id int4) RETURNS TEXT
AS $$

class Node(object):

    def __init__(self, name, id=None, children=[]):
        self.name = name
        self.id = id
        self.children = children

    def to_json(self, out):
        out.append('{"name":"%s","id":%d' % (self.name, self.id))
        if self.children:
            out.append(',"children":[')
            first = True
            for child in self.children:
                if first:
                    first = False
                else:
                    out.append(",")
                child.to_json(out)
            out.append("]")
        out.append("}")

    def recursive_build(self, mid, children, parent_name="master"):
        select = ("SELECT id FROM %s WHERE %s_id = %d"
                  % (self.name, parent_name, mid))
        ids = [r['id'] for r in plpy.execute(select)]
        for id in ids:
            sub_children = []
            node = Node(self.name, id, sub_children)
            name = self.name[:-4] # cut off trailing '_tbl'
            for child in self.children:
                child.recursive_build(id, sub_children, name)
            children.append(node)

TREE_STRUCTURE = Node('master_tbl', children = [
    Node('rg_kontaktdaten_tbl'),
    Node('rg_kontaktdaten_sorgeberechtigte_tbl'),
    Node('rg_institution_tbl'),
    Node('rg_sprache_zuhause_weitere_tbl'),
    Node('rg_kinder_tbl'),
    Node('rg_biographie2_tbl'),
    Node('rg_weitere_sprache_tbl'),
    Node('rg_ziele_tbl')
])


children = []
tree = Node(TREE_STRUCTURE.name, case_id, children)
for child in TREE_STRUCTURE.children:
    child.recursive_build(case_id, children)
out = []
tree.to_json(out)
return "".join(out)

$$ LANGUAGE plpythonu EXTERNAL SECURITY DEFINER;
ALTER FUNCTION compute_case_structure(int4) OWNER TO :adm_ka_owner;


SET ROLE :adm_ka_owner;
