BEGIN;

-- 1.0pre5 -> jmd-1.0pre6
-- Deadlock bei updates von case_structur vermeiden (mpuls/issue1145)

INSERT INTO db_history (beschreibung, zeitpunkt)
VALUES ('jmd-1.0pre6/04-case-structure.sql', current_timestamp);


-- DROP FUNCTION compute_case_structure(int4);
CREATE OR REPLACE FUNCTION compute_case_structure(case_id int4) RETURNS TEXT
AS $$

class Node(object):

    def __init__(self, name, id=None, children=[]):
        self.name = name
        self.id = id
        self.children = children

    def to_json(self, out):
        out.append('{"name":"%s","id":%d' % (self.name, self.id))
        if self.children:
            out.append(',"children":[')
            first = True
            for child in self.children:
                if first:
                    first = False
                else:
                    out.append(",")
                child.to_json(out)
            out.append("]")
        out.append("}")

    def recursive_build(self, mid, children):
        select = "SELECT id FROM %s WHERE master_id = %d" % (self.name, mid)
        ids = [r['id'] for r in plpy.execute(select)]
        for id in ids:
            sub_children = []
            node = Node(self.name, id, sub_children)
            for child in self.children:
                child.recursive_build(id, sub_children)
            children.append(node)

TREE_STRUCTURE = Node('master_tbl', children=[
    Node('rg_institution_tbl'),
    Node('rg_kontaktdaten_tbl'),
    Node('rg_kontaktdaten_sorgeberechtigte_tbl'),
    Node('rg_kinder_tbl'),
    Node('rg_biographie2_tbl'),
    Node('rg_weitere_sprache_tbl'),
    Node('rg_ziele_tbl')
])

children = []
tree = Node(TREE_STRUCTURE.name, case_id, children)
for child in TREE_STRUCTURE.children:
    child.recursive_build(case_id, children)
out = []
tree.to_json(out)
return "".join(out)

$$ LANGUAGE plpythonu EXTERNAL SECURITY DEFINER;


-- DROP FUNCTION get_case_structure(int4);
CREATE OR REPLACE FUNCTION get_case_structure(case_id int4) RETURNS TEXT
AS $$

r = plpy.execute("SELECT modified, cache FROM case_structure "
                 "WHERE master_id = %d" % case_id, 1)

if not r:
    return ""
r = r[0]

if r['modified']: # need refresh
    structure = plpy.execute("SELECT compute_case_structure(%d) as s;"
                             % case_id, 1)[0]["s"]
    plan = plpy.prepare("UPDATE case_structure SET modified = false, "
                        "cache = $1 WHERE master_id = $2", ["text", "int4"])
    plpy.execute(plan, [structure, case_id])
    return structure

return r['cache']

$$ LANGUAGE plpythonu EXTERNAL SECURITY DEFINER;
ALTER FUNCTION get_case_structure(int4) OWNER TO :adm_ka_owner;


SET ROLE :db_adm;


CREATE OR REPLACE FUNCTION rg_institution_tbl_insert_func() RETURNS TRIGGER AS $$
BEGIN
    UPDATE case_structure
    SET modified = FALSE, cache = compute_case_structure(master_id)
    WHERE master_id = NEW.master_id;
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION rg_institution_tbl_delete_func() RETURNS TRIGGER AS $$
BEGIN
    UPDATE case_structure
    SET modified = FALSE, cache = compute_case_structure(master_id)
    WHERE master_id = OLD.master_id;
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;

DROP TRIGGER rg_institution_tbl_delete_trigger ON rg_institution_tbl CASCADE;
CREATE TRIGGER rg_institution_tbl_delete_trigger AFTER DELETE ON rg_institution_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_institution_tbl_delete_func();


CREATE OR REPLACE FUNCTION rg_kontaktdaten_tbl_insert_func() RETURNS TRIGGER AS $$
BEGIN
    UPDATE case_structure
    SET modified = FALSE, cache = compute_case_structure(master_id)
    WHERE master_id = NEW.master_id;
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION rg_kontaktdaten_tbl_delete_func() RETURNS TRIGGER AS $$
BEGIN
    UPDATE case_structure
    SET modified = FALSE, cache = compute_case_structure(master_id)
    WHERE master_id = OLD.master_id;
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;

DROP TRIGGER rg_kontaktdaten_tbl_delete_trigger ON rg_kontaktdaten_tbl CASCADE;
CREATE TRIGGER rg_kontaktdaten_tbl_delete_trigger AFTER DELETE ON rg_kontaktdaten_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_kontaktdaten_tbl_delete_func();


CREATE OR REPLACE FUNCTION rg_kontaktdaten_sorgeberechtigte_tbl_insert_func() RETURNS TRIGGER AS $$
BEGIN
    UPDATE case_structure
    SET modified = FALSE, cache = compute_case_structure(master_id)
    WHERE master_id = NEW.master_id;
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION rg_kontaktdaten_sorgeberechtigte_tbl_delete_func() RETURNS TRIGGER AS $$
BEGIN
    UPDATE case_structure
    SET modified = FALSE, cache = compute_case_structure(master_id)
    WHERE master_id = OLD.master_id;
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;

DROP TRIGGER rg_kontaktdaten_sorgeberechtigte_tbl_delete_trigger ON rg_kontaktdaten_sorgeberechtigte_tbl CASCADE;
CREATE TRIGGER rg_kontaktdaten_sorgeberechtigte_tbl_delete_trigger AFTER DELETE ON rg_kontaktdaten_sorgeberechtigte_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_kontaktdaten_sorgeberechtigte_tbl_delete_func();


CREATE OR REPLACE FUNCTION rg_kinder_tbl_insert_func() RETURNS TRIGGER AS $$
BEGIN
    UPDATE case_structure
    SET modified = FALSE, cache = compute_case_structure(master_id)
    WHERE master_id = NEW.master_id;
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION rg_kinder_tbl_delete_func() RETURNS TRIGGER AS $$
BEGIN
    UPDATE case_structure
    SET modified = FALSE, cache = compute_case_structure(master_id)
    WHERE master_id = OLD.master_id;
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;

DROP TRIGGER rg_kinder_tbl_delete_trigger ON rg_kinder_tbl CASCADE;
CREATE TRIGGER rg_kinder_tbl_delete_trigger AFTER DELETE ON rg_kinder_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_kinder_tbl_delete_func();


CREATE OR REPLACE FUNCTION rg_biographie2_tbl_insert_func() RETURNS TRIGGER AS $$
BEGIN
    UPDATE case_structure
    SET modified = FALSE, cache = compute_case_structure(master_id)
    WHERE master_id = NEW.master_id;
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION rg_biographie2_tbl_delete_func() RETURNS TRIGGER AS $$
BEGIN
    UPDATE case_structure
    SET modified = FALSE, cache = compute_case_structure(master_id)
    WHERE master_id = OLD.master_id;
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;

DROP TRIGGER rg_biographie2_tbl_delete_trigger ON rg_biographie2_tbl CASCADE;
CREATE TRIGGER rg_biographie2_tbl_delete_trigger AFTER DELETE ON rg_biographie2_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_biographie2_tbl_delete_func();


CREATE OR REPLACE FUNCTION rg_weitere_sprache_tbl_insert_func() RETURNS TRIGGER AS $$
BEGIN
    UPDATE case_structure
    SET modified = FALSE, cache = compute_case_structure(master_id)
    WHERE master_id = NEW.master_id;
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION rg_weitere_sprache_tbl_delete_func() RETURNS TRIGGER AS $$
BEGIN
    UPDATE case_structure
    SET modified = FALSE, cache = compute_case_structure(master_id)
    WHERE master_id = OLD.master_id;
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;

DROP TRIGGER rg_weitere_sprache_tbl_delete_trigger ON rg_weitere_sprache_tbl CASCADE;
CREATE TRIGGER rg_weitere_sprache_tbl_delete_trigger AFTER DELETE ON rg_weitere_sprache_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_weitere_sprache_tbl_delete_func();


CREATE OR REPLACE FUNCTION rg_ziele_tbl_insert_func() RETURNS TRIGGER AS $$
BEGIN
    UPDATE case_structure
    SET modified = FALSE, cache = compute_case_structure(master_id)
    WHERE master_id = NEW.master_id;
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION rg_ziele_tbl_delete_func() RETURNS TRIGGER AS $$
BEGIN
    UPDATE case_structure
    SET modified = FALSE, cache = compute_case_structure(master_id)
    WHERE master_id = OLD.master_id;
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;

DROP TRIGGER rg_ziele_tbl_delete_trigger ON rg_ziele_tbl CASCADE;
CREATE TRIGGER rg_ziele_tbl_delete_trigger AFTER DELETE ON rg_ziele_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_ziele_tbl_delete_func();


-- vorhandene case_structure Eintrge die als veraltet markiert sind
-- (modified=true) ausfllen

UPDATE case_structure SET
    modified = FALSE, cache = compute_case_structure(master_id)
WHERE modified;

COMMIT;
