# -*- coding: utf-8 -*-
#
# Copyright 2015 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls JMD
#
# mpuls JMD is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls JMD is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls JMD. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls JMD has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.

from mpulsweb.controllers.case_overview import \
    CaseOverviewController as MpulsCaseOverviewController, \
    BoolConvertValidator, get_session_search, put_session_search

from mpulsweb.lib.helper.searchfield import extra_info
from mpulsweb.lib.searchexpr import And
from mpulsweb.lib.helpers import url
from mpulsweb.lib.helper.searchfield import SearchFieldDescription
from mpulsweb.model.agencysettings import Agency

from jmdweb.lib.searchexpr import BoolGroup

from pylons.controllers.util import redirect

from pylons import app_globals as g


BOOL_GROUP_NAME = "group-32"
# Group-32 is the Bezugspersonen group. Which is annoying to
# be searched in a way that requires a great deal of formed
# tree knowlege for implementation.
#
# E.g. bezugsperson_q is "Keine Bezugsperson" and thus needs
# to be treated like there is no Bezugsperson set.


class CaseOverviewController(MpulsCaseOverviewController):

    """Override of base flexible overview to add special handling
    for bezugsperson fields.
    """

    def _get_field_filter(self):
        jmd2start_fields = ["zielgruppe", "aufenthaltspapier"]
        if Agency().get_value("jmd2start"):
            jmd2start_fields = []
        return (super(CaseOverviewController, self)._get_field_filter()
                + [e.getName()
                   for e in g.formedTree.findByName(BOOL_GROUP_NAME).children]
                + jmd2start_fields)

    def _get_additional_fields(self):
        bz_group = g.formedTree.findByName(BOOL_GROUP_NAME)
        return [dict(description=bz_group.get_title(),
                     name=bz_group.getName())]

    def _get_search_field_description(self, fieldname):
        if fieldname != BOOL_GROUP_NAME:
            return super(CaseOverviewController,
                         self)._get_search_field_description(fieldname)
        bz_group = g.formedTree.findByName(BOOL_GROUP_NAME)
        return SearchFieldDescription(u"group_bool", bz_group.get_title())

    def _add_field(self, desc):
        if desc != g.formedTree.findByName(BOOL_GROUP_NAME).get_title():
            return super(CaseOverviewController, self)._add_field(desc)
        ast = get_session_search()
        put_session_search(And(ast.exprs + [BoolGroup(BOOL_GROUP_NAME, None)]))
        redirect(url(controller='case_overview', action='overview'))

    def _update_ast_field(self, expr):
        if not isinstance(expr, BoolGroup):
            return super(CaseOverviewController, self)._update_ast_field(expr)

        value, x, y = BoolConvertValidator(expr.groupname).validate()
        return BoolGroup(expr.groupname, value)
