# -*- coding: utf-8 -*-

#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
# Sascha L. Teichmann <teichmann@intevation.de>
# Roland Geider <roland.geider@intevation.de>

import logging
import formencode

from pylons import request, tmpl_context as c, app_globals as g
from pylons.controllers.util import redirect

from mpulsweb.lib.translation import _

from mpulsweb.lib.base import BaseController, render, session, render_htmlfill,\
    restrict_to_POST, abort_unless_POST
from mpulsweb.lib.helpers import url
from mpulsweb.lib.security import checkRole, getDbName
from mpulsweb.lib.validators import ValidatorStateObject, NewUserGroupForm, \
    EditUserGroupForm, NewUserForm, EditUserForm, DeleteUserHelperForm
from mpulsweb.model.user import UserListObject, UserObject, UserGroupList, \
    UserGroup, UserGroupFactory
from mpulsweb.model.case import MpulsCaseOverview, MpulsCaseBundle
from mpulsweb.model.agency import MpulsAgencyOverview


log = logging.getLogger(__name__)


class AdministrationController(BaseController):

    @checkRole('admin')
    def index(self):
        return self.overviewUser()

    # USER
    @checkRole('admin')
    def overviewUser(self):
        list = UserListObject()
        c.user_overview = list.getUserList()
        return render('/administration/overview_user.mako')

    @checkRole('admin')
    def newUser(self):
        c.form_errors = {}
        c.form_result = {}
        c.uo = UserObject()

        # Load the agency list if application is running as evaluation server
        if g.mpuls_config.is_enabled('module', 'agency-overview'):
            agency_overview = MpulsAgencyOverview()
            agency_overview.search('sort_field:fkz;sort_order:asc')
            c.agency_list = agency_overview.getDatasets()

        return render('/administration/new_user.mako')

    @checkRole('admin')
    @restrict_to_POST
    def newUserAction(self):
        validator = self._get_NewUserForm()
        c.user_password = None
        try:
            agency = getDbName()
            state = ValidatorStateObject()
            state.agency = agency
            c.form_result = validator.to_python(request.params, state)
            c.uo = UserObject()
            # Load the agency list if application is running as
            # evaluation server
            if g.mpuls_config.is_enabled('module', 'agency-overview'):
                agency_overview = MpulsAgencyOverview()
                agency_overview.search('sort_field:fkz;sort_order:asc')
                c.agency_list = agency_overview.getDatasets()
            # Set initial data which is needed to create a new user
            login = c.form_result.get('login', '')
            role = c.form_result.get('role', '')
            c.uo.setData(c.form_result)
            # create user
            c.user_password = c.uo.createLogin(login, role, agency)
            self._after_new_user(c.form_result, c.uo)
            return render('/administration/new_user_result.mako')
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            form = render('/administration/new_user.mako')
            return render_htmlfill(form, c.form_result, c.form_errors,
                                   auto_insert_errors=False)

    def _get_NewUserForm(self):
        '''Can override it'''
        return NewUserForm()

    def _after_new_user(self, form_result, user):
        '''Make custom code possible.'''
        pass

    @checkRole('admin')
    def showUser(self, id=None):
        id = self._checkInt(id)
        c.uo = UserObject(id)
        c.vuo = UserGroup(c.uo.standin)
        self._show_user_extra_content(c)
        # Load the agency list if application is running as evaluation server
        if g.mpuls_config.is_enabled('module', 'agency-overview'):
            agency_overview = MpulsAgencyOverview()
            agency_overview.search('sort_field:fkz;sort_order:asc')
            c.agency_list = agency_overview.getDatasets()
        return render('/administration/show_user.mako')

    def _show_user_extra_content(self, content):
        '''Make custom code possible.'''
        pass

    @checkRole('admin')
    def editUser(self, id=None):
        id = self._checkInt(id)
        c.form_errors = {}
        c.form_result = {}
        c.uo = UserObject(id)
        self._edit_user_extra_content(c)
        # Load the agency list if application is running as evaluation server
        if g.mpuls_config.is_enabled('module', 'agency-overview'):
            agency_overview = MpulsAgencyOverview()
            agency_overview.search('sort_field:fkz;sort_order:asc')
            c.agency_list = agency_overview.getDatasets()
        return render('/administration/edit_user.mako')

    def _edit_user_extra_content(self, content):
        '''Make custom code possible.'''
        pass

    @checkRole('admin')
    @restrict_to_POST
    def editUserAction(self):
        validator = self._get_EditUserForm()
        try:
            uid = request.params['uid']
            c.uo = UserObject(uid)
            # Load the agency list if application is running as
            # evaluation server
            if g.mpuls_config.is_enabled('module', 'agency-overview'):
                agency_overview = MpulsAgencyOverview()
                agency_overview.search('sort_field:fkz;sort_order:asc')
                c.agency_list = agency_overview.getDatasets()
            form_result = validator.to_python(request.params)
            c.uo.setData(form_result)
            self._after_edit_user(form_result, c.uo)
            c.uo.store()
            c.dialog_title = _('User configs saved!')
            c.dialog_text = _("Please click okay to continue.")
            c.url_ok = url(controller='administration',
                           action='overviewUser')
            return render('/administration/dialogs/success_edit_user.mako')
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            return render('/administration/edit_user.mako')

    def _get_EditUserForm(self):
        return EditUserForm()

    def _after_edit_user(self, form_result, user):
        pass

    @checkRole('admin')
    def resetUserPassword(self, id=None, confirmed='0'):
        id = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        c.uo = UserObject(id)
        if confirmed:
            abort_unless_POST()
            c.new_password = c.uo.resetPassword()
            return render('/administration/reset_password.mako')
        else:
            c.context = "../main.mako"
            c.dialog_title = _('Reset user password?')
            c.dialog_text = "%s %s, %s" % (_('Confirm reset user password:'),
                                           c.uo.last_name, c.uo.first_name)

            c.url_yes = url(controller='administration',
                            action='resetUserPassword', id=id, confirmed=1)
            c.url_no = url(controller='administration',
                           action='showUser', id=id)
            return render('/administration/dialogs/confirm_resetpassword.mako')

    @checkRole('admin')
    def deleteUser(self, id, confirmed='0'):
        id = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        uo = UserObject(id)
        # Make sure that the user can not delete himself
        if uo.id == session.get('USER_AUTHORIZED').id:
            c.dialog_title = _('User not deleted!')
            c.dialog_text = _('Not possible to delete current login user.'
                              ' Click on okay to continue.')
            c.url_ok = url(controller='administration', action='overviewUser')
            return render('/administration/dialogs/failed_delete_user.mako')

        if confirmed:
            abort_unless_POST()
            # Check if there are cases for which the user is the editor. If
            # he is an editor then raise a dialog which allows the user to
            # transfer the cases.
            overview = MpulsCaseOverview()
            overview.search({"editor": uo.id})
            if overview.getDatasets():
                list = UserListObject()
                filter = [u.id for u in list.getAdminList()]
                filter.append(uo.id)  # the user we want to delete
                c.user_list = list.getUserList(filter)
                c.cases = overview
                log.debug(c.cases)
                c.delete_user_id = id
                c.statemachine = g.case_factory.get_state_machine()
                return render('/administration/delete_user_helper.mako')
            else:
                self._before_delete_user(id)
                uo.delete()
                c.success_for = _('User deleted!')
                c.success_text = _('User sucessfully deleted. '
                                   'Click okay to continue.')
                c.url_ok = url(controller='administration',
                               action='overviewUser')
                return render('/administration/dialogs/'
                              'success_delete_user.mako')

        else:
            c.delete_user_id = id
            c.context = "../main.mako"
            c.confirm_for = _('Delete user?')
            c.question = (_('Please confirm to delete user '
                            '<strong>%s, %s</strong> (login: %s).')
                          % (uo.last_name, uo.first_name, uo.login))
            c.url_yes = url(controller='administration',
                            action='deleteUser', id=id, confirmed=1)
            c.url_no = url(controller='administration',
                           action='overviewUser')
            return render('/administration/dialogs/confirm_deleteuser.mako')

    def _before_delete_user(self, id):
        """
        Application specific actions to perform before actually deleting
        the user. For example, applications with a structure part can
        remove the rights to the user to access it.
        """
        pass

    @checkRole('admin')
    @restrict_to_POST
    def deleteUserHelper(self):
        validator = DeleteUserHelperForm()
        all_cases = MpulsCaseOverview()
        c.notification_for = _('Assign case document')
        c.form_errors = {}
        c.form_result = {}

        if 'editor' in request.params:
            try:
                form_result = validator.to_python(request.params)
                case_bundle = MpulsCaseBundle(form_result.get('case_id', []))
                new_editor = UserObject(form_result.get('editor'))
                del_user = UserObject(form_result.get('user_id'))
                all_cases.search({"editor": del_user.id})
                case_bundle.setEditor(form_result.get('editor'))
                rest = all_cases.numDatasets() - case_bundle.numDatasets()
                c.url_ok = url(controller='administration',
                               action='deleteUser',
                               id=form_result.get('user_id'))
                c.success_for = _('Case document assigned!')
                if case_bundle.numDatasets() == 0:
                    c.success_for = _('More case documents exist!')
                    c.success_text = (_(u'No case document assigned. '
                                        u'%s, %s has more case documents.'
                                        u' Please click on OK to continue.')
                                      % (del_user.first_name,
                                         del_user.last_name))

                elif rest > 0:
                    c.success_for = _('Case document assigned. '
                                      'More case documents exist!')
                    c.success_text = (_(u'Case document assigned to %s, %s.'
                                        u' %s, %s has more case documents.'
                                        u' Please click on OK to continue.')
                                      % (new_editor.first_name,
                                         new_editor.last_name,
                                         del_user.first_name,
                                         del_user.last_name))
                else:
                    c.success_for = _('Case document assigned!')
                    c.success_text = (_('Case document assigned to %s, %s.'
                                        ' Please click on OK to delete %s, %s.')
                                      % (new_editor.first_name,
                                         new_editor.last_name,
                                         del_user.first_name,
                                         del_user.last_name))
                return render('/administration/dialogs/'
                              'notificate_delete_user_helper.mako')
            except formencode.Invalid:
                return self.deleteUser(form_result.get('user_id'), '1')
        else:
            redirect(url(controller="administration", action='overviewUser'))

    # USERGROUPS
    @checkRole('admin')
    def overviewUserGroups(self):
        # Get gid of usergroups so we can filter them out.
        user = UserListObject()
        filter = [u.gid for u in user.getUserList()]

        list = UserGroupList()
        c.group_overview = list.getGroupList(filter)
        return render('/administration/overview_usergroups.mako')

    @checkRole('admin')
    def newUserGroup(self):
        c.form_errors = {}
        c.form_result = {}
        return render('/administration/new_usergroup.mako')

    @checkRole('admin')
    @restrict_to_POST
    def newUserGroupAction(self):
        factory = UserGroupFactory()
        validator = NewUserGroupForm()
        try:
            c.form_result = validator.to_python(request.params)
            c.ugo = factory.create(c.form_result.get('name'))
            c.dialog_title = _('Create user group!')
            c.dialog_text = _(u'User group successfully created. '
                              u'Please click on OK to continue to the'
                              u' user group overview')
            c.url_ok = url(controller='administration',
                           action='overviewUserGroups')
            return render('/administration/dialogs/success_new_usergroup.mako')
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            form = render('/administration/new_usergroup.mako')
            return render_htmlfill(form, c.form_result, c.form_errors,
                                   auto_insert_errors=False)
        except Exception, e:
            log.exception(e)
            c.dialog_title = _('Failed to create user group!')
            c.dialog_text = _('Failed to create the user group.'
                              ' Please click on OK to continue in the'
                              ' user overview.')
            c.url_ok = url(controller='administration',
                           action='overviewUserGroups')
            return render('/administration/dialogs/failed_new_usergroup.mako')

    @checkRole('admin')
    def showUserGroup(self, id=None):
        factory = UserGroupFactory()
        id = self._checkInt(id)
        group = factory.load(id)
        groupmember = group.getUser()
        c.group = group
        c.groupmember = groupmember
        return render('/administration/show_usergroup.mako')

    def _render_edit_user_group(self, group, form_result, form_errors=None):
        c.form_errors = form_errors or {}
        c.form_result = form_result
        c.group = group

        userlist = UserListObject()
        admins = userlist.getAdminList()
        c.userlist = userlist.getUserList(filter=[u.id for u in admins])
        form = render('/administration/edit_usergroup.mako')
        return render_htmlfill(form, c.form_result, c.form_errors,
                               auto_insert_errors=False)

    @checkRole('admin')
    def editUserGroup(self, id=None):
        id = self._checkInt(id)
        factory = UserGroupFactory()
        group = factory.load(id)
        form_result = {'name': group.name,
                       'id': group.id,
                       'standin': [user.id
                                   for user in group.getUser().getUserList()]}
        return self._render_edit_user_group(group, form_result)

    @checkRole('admin')
    @restrict_to_POST
    def editUserGroupAction(self):
        factory = UserGroupFactory()

        validator = EditUserGroupForm()
        try:
            form_result = validator.to_python(request.params)
        except formencode.Invalid, error:
            result = error.value
            return self._render_edit_user_group(factory.load(result.get('id')),
                                                result, error.error_dict)
        else:
            group = factory.load(form_result["id"])
            group.setData(form_result)
            group.setUser(form_result["standin"])
            group.store()
            c.group = group
            c.dialog_title = _('Edit user group!')
            c.dialog_text = _('The user group was successfully edited. '
                              'Please click on OK '
                              'to continue in the user group overview.')
            c.url_ok = url(controller='administration',
                           action='overviewUserGroups')
            return render('/administration/dialogs/success_edit_usergroup.mako')

    @checkRole('admin')
    def deleteUserGroup(self, id, confirmed='0'):
        id = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        factory = UserGroupFactory()
        try:
            ugo = factory.load(id)
            if confirmed:
                abort_unless_POST()
                ugo.delete()
                c.dialog_title = _('User group deleted!')
                c.dialog_text = _('The user group was successfully deleted.'
                                  ' Please click on OK to continue in the'
                                  ' user group overview.')
                c.url_ok = url(controller='administration',
                               action='overviewUserGroups')
                return render('/administration/dialogs/'
                              'success_deletegroup_user.mako')
            else:
                c.context = "../main.mako"
                c.dialog_title = _('Delete user group?')
                c.dialog_text = _('Do you want to delete the user group and '
                                  'lose the data?')
                c.url_yes = url(controller='administration',
                                action='deleteUserGroup', id=ugo.id,
                                confirmed=1)
                c.url_no = url(controller='administration',
                               action='overviewUserGroups')
                return render('/administration/dialogs/'
                              'confirm_deleteusergroup.mako')
        except Exception, e:
            log.exception(e)
            c.dialog_title = _('Failed to delete the user group!')
            c.dialog_text = _('Deletion of the user group failed. '
                              'Please click on OK to continue'
                              ' in the user group overview.')
            c.url_ok = url(controller='administration',
                           action='overviewUserGroups')
            return render('/administration/dialogs/failed_new_usergroup.mako')


# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
