# -*- coding: utf-8 -*-
import logging

from pylons import tmpl_context as c, app_globals as g
from pylons.controllers.util import abort

from mpulsweb.lib.translation import _
from mpulsweb.lib.base import BaseController, h, render, request, \
    restrict_to_POST
from mpulsweb.lib.security import checkRole, check_state
from mpulsweb.model.case import OPEN_STATE, CLOSED_STATE, \
    OUTSTANDING_DELETION_STATE, OUTSTANDING_ANONYMISATION_STATE, \
    ANONYMISED_STATE

log = logging.getLogger(__name__)


class PhaseController(BaseController):

    @checkRole(('admin', 'cm'))
    def index(self, id):
        return self.overview(id)

    @checkRole(('admin', 'cm'))
    def overview(self, id):
        id = self._checkInt(id)
        case = self._loadCase(id)
        if not check_state(case, {'admin': [OPEN_STATE, CLOSED_STATE,
                                            OUTSTANDING_DELETION_STATE,
                                            OUTSTANDING_ANONYMISATION_STATE,
                                            ANONYMISED_STATE],
                                  'cm': [OPEN_STATE, CLOSED_STATE]}):
            abort(401)

        sm = g.case_factory.get_state_machine()
        c.statemachine = sm
        c.case = case
        # Only show buttons to set the phase if
        c.show_actions = sm.get_transition_mode() != sm.TRANSITION_MODE_FULLAUTO
        return render('/phase/phase.mako')

    @checkRole(('cm'))
    @restrict_to_POST
    def setPhase(self, id, value):
        case = self._loadCase(id)

        # check phase
        sm = g.case_factory.get_state_machine()
        if not sm.is_reachable(case, value):
            abort(403)

        # Set phase
        new_state = sm.get(value)
        case.set_phase_state(new_state)

        # After setting the phase. Show print-dialog
        c.dialog_title = _("Phase changed!")
        c.dialog_text = _("The phase could be changed successfully."
                          " Please follow the shown steps to reach the"
                          " phase overview.")
        c.url_ok = h.url(controller='phase', action='overview', id=case.id)
        return render('/phase/dialogs/success_set_phase.mako')

    @checkRole(('cm'))
    @restrict_to_POST
    def editAction(self):
        newphase = None

        # TODO: This is ugly. The id of the phase to be set is submitted as key
        # (intvalue)) of the submitbutton. So to find the right value, try to
        # convert all keys to int, which should only work for the
        # submitbuttons.
        for newphase in request.params.iterkeys():
            try:
                newphase = int(newphase)
                break
            except:
                # convert failed, seems not to be the submit button.
                # Continue...
                pass
        return self.setPhase(request.params.get('id'), newphase)
