# -*- coding: utf-8 -*-
#
# (c) 2014 by Intevation GmbH
# This is Free software under the GPLv3.
# See LICENSE comming with the source of 'mpuls'
# for details.
#
# author: Bjoern Ricks <bjoern.ricks@intevation.de>
#

import logging
import os
import sys

from mpulsweb.lib.commands import update_commands, find_command, \
    help_command_list


log = logging.getLogger(__name__)


def import_module(name):
    """
    Imports a module with name

    name must be a dottet string like myapp.package.module
    """
    try:
        __import__(name)
        return sys.modules[name]
    except Exception, e:
        log.error("could not import %s. Error %s" % (name, e))


def find_commands(commands_dir):
    """
    Returns a list of command modules from the commands_dir or an empty list if
    no command modules are found.
    """
    try:
        return [f[:-3] for f in os.listdir(commands_dir)
                if not f.startswith('_') and f.endswith('.py')]
    except OSError:
        return []


def load_command_module(mpuls_app_name, name):
    """
    Loads the command module
    """
    import_module('%s.commands.%s' % (mpuls_app_name, name))


def load_app_commands(mpuls_app_name):
    """
    Loads all commands from the command module for the mpuls_app
    """
    cmd_names = find_commands(os.path.join(mpuls_app_name, "commands"))
    load_commands(mpuls_app_name, cmd_names)


def load_commands(mpuls_app_name, cmd_names):
    for name in cmd_names:
        load_command_module(mpuls_app_name, name)


def load_base_commands(basedir):
    cmd_names = find_commands(os.path.join(basedir, "mpulsweb", "commands"))
    load_commands("mpulsweb", cmd_names)


def execute_from_cli(*apps, **kwargs):
    """
    Executes commands

    apps should be the module names where to load the commands from
    arguments is the arguments list parsed by the command interpreter
    db_admin_path is the path to the mpulsdbadmin.py script
    """
    arguments = kwargs.pop("args", None)
    db_admin_path = kwargs.pop("db_admin_path", None)

    if kwargs:
        for key in kwargs.keys():
            raise TypeError("Unkown argument %s for execute_from_cli()" % key)

    if arguments is None:
        arguments = sys.argv

    f_dir = os.path.dirname(__file__)
    basedir = os.path.join(f_dir, os.pardir, os.pardir)

    if db_admin_path is None:
        db_admin_path = os.path.join(basedir, os.pardir, "db-admin")

    sys.path.append(db_admin_path)

    try:
        import mpulsdbadm
        cmd_map = mpulsdbadm.get_command_map()
        update_commands(cmd_map)
    except:
        pass

    load_base_commands(basedir)

    for app_name in apps:
        load_app_commands(app_name)

    main(arguments)


def main(args):
    if not len(args) > 1:
        help_command_list(args[0])
        sys.exit(1)

    command_name = args[1]

    command = find_command(command_name)

    if command is None:
        print >>sys.stderr, "Unknown command: %r" % (command_name,)
        help_command_list(args[0])
        sys.exit(1)

    ret = command(os.path.basename(args[0]), args[1:])
    if ret:
        sys.exit(ret)
    return 0
