SET ROLE :adm_ka_owner;
BEGIN;

-- anonymized_uuids
-- Liste von UUIDs, die bereits anonymisiert wurden. Die UUIDs
-- in den Akten selbst werden beim Anonymisieren aus
-- datenschutzrechtlichen Gruenden geloescht.
CREATE TABLE anonymized_uuids (
        id      serial   PRIMARY KEY,
    uuid_id CHAR(36) UNIQUE NOT NULL
);
REVOKE ALL ON anonymized_uuids FROM public;

-- Testet, ob eine UUID schon anonmisiert wurde.
CREATE OR REPLACE FUNCTION is_uuid_anonymized(nuuid char(36))
    RETURNS BOOL AS
$$
DECLARE
    anon BOOL;
BEGIN
    IF nuuid is NULL THEN
        RETURN TRUE;
    END IF;
    SELECT INTO anon TRUE FROM anonymized_uuids WHERE uuid_id = nuuid;
    IF anon THEN
        RETURN TRUE;
    END IF;
    RETURN FALSE;
END;
$$ LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER;

REVOKE ALL ON FUNCTION is_uuid_anonymized(char(36)) FROM public;
GRANT EXECUTE ON FUNCTION is_uuid_anonymized(char(36)) TO GROUP :admin_group;
GRANT EXECUTE ON FUNCTION is_uuid_anonymized(char(36)) TO GROUP :cm_group;

-- Loescht die UUID in einer Akte und traegt sie in die Liste der
-- anonymisierten UUIDs ein.
CREATE OR REPLACE FUNCTION anonymize_uuid(mid int)
RETURNS void AS
$$
DECLARE
    nuuid char(36);
BEGIN
    SELECT INTO nuuid uuid_id FROM master_tbl_view WHERE id = mid;
    IF NOT is_uuid_anonymized(nuuid) THEN
        INSERT INTO anonymized_uuids (uuid_id) VALUES (nuuid);
    END IF;
    UPDATE master_tbl SET uuid_id = NULL WHERE id = mid;
END;
$$ LANGUAGE 'plpgsql' VOLATILE SECURITY DEFINER;

REVOKE ALL ON FUNCTION anonymize_uuid(int) FROM public;
GRANT EXECUTE ON FUNCTION anonymize_uuid(int) TO GROUP :admin_group;

-- Funktionen, die für die Anonymisierung der Fallakten genutzt werden.
CREATE OR REPLACE FUNCTION shorten_value(myvalue varchar, len integer)
RETURNS varchar
LANGUAGE 'plpgsql' STABLE AS
$$
BEGIN
    return substring(myvalue, 1, len);
END;
$$;

REVOKE ALL ON FUNCTION shorten_value(varchar, integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION shorten_value(varchar, integer) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION shorten_value(varchar, integer) TO GROUP :admin_group;

CREATE OR REPLACE FUNCTION hash_value(myvalue varchar, myvalue2 varchar)
RETURNS varchar
LANGUAGE 'plpgsql' STABLE AS
$$
BEGIN
    return md5(coalesce(myvalue, '') || coalesce(myvalue2, ''));
END;
$$;

REVOKE ALL ON FUNCTION hash_value(varchar, varchar) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION hash_value(varchar, varchar) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION hash_value(varchar, varchar) TO GROUP :admin_group;

CREATE OR REPLACE FUNCTION get_age(myvalue date) 
RETURNS date 
LANGUAGE 'plpgsql' STABLE AS 
$$
BEGIN
    return date_trunc('year', myvalue::timestamp);
END;
$$;

REVOKE ALL ON FUNCTION get_age(date) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION get_age(date) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION get_age(date) TO GROUP :admin_group;

CREATE OR REPLACE FUNCTION check_bool(myvalue integer, mybool integer)
RETURNS integer
LANGUAGE 'plpgsql' STABLE AS
$$
BEGIN
    IF mybool = 1 THEN
        return myvalue;
    ELSE
        return NULL::integer;
    END IF;
END;
$$;

REVOKE ALL ON FUNCTION check_bool(integer, integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION check_bool(integer, integer) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION check_bool(integer, integer) TO GROUP :admin_group;

CREATE OR REPLACE FUNCTION check_bool(myvalue varchar, mybool integer)
RETURNS varchar
LANGUAGE 'plpgsql' STABLE AS
$$
BEGIN
    IF mybool = 1 THEN
        return myvalue;
    ELSE
        return NULL::varchar;
    END IF;
END;
$$;

REVOKE ALL ON FUNCTION check_bool(varchar, integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION check_bool(varchar, integer) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION check_bool(varchar, integer) TO GROUP :admin_group;

-- Wrapper Funktion die alle noetigen Funktionen zum Anonymisieren einer
-- Fallakte aufruft.
CREATE OR REPLACE FUNCTION anonymize_case_wrapper(mid integer)
RETURNS void AS
$$
BEGIN
        PERFORM anonymize_case(mid);
        PERFORM delete_meta_ds(mid);
        PERFORM delete_all_fall_dokumente(mid);
        PERFORM delete_all_fall_termine(mid);
        PERFORM anonymize_logbuch(mid);
END;
$$ LANGUAGE plpgsql VOLATILE SECURITY DEFINER;

REVOKE ALL     ON FUNCTION anonymize_case_wrapper(integer) FROM PUBLIC;
GRANT  EXECUTE ON FUNCTION anonymize_case_wrapper(integer) TO GROUP :admin_group;


COMMIT;
