SET ROLE :adm_ka_owner;
BEGIN;

CREATE TABLE ka_dokumente_tbl
(
        id serial NOT NULL,
        uuid_id CHAR(36) UNIQUE,
        master_id int4 REFERENCES master_tbl(id) ON DELETE CASCADE,
        groesse int4,
        mime varchar(256),
        daten bytea,
        name varchar(256),
        datum date,
        CONSTRAINT ka_dokumente_tbl_pkey PRIMARY KEY (id)
);

-- Funktionen, zum Anlegen und Löschen von Fallanhängen 
CREATE OR REPLACE FUNCTION create_fall_dokument_ds(mid integer, nuuid char(36), filename varchar) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
        did integer;
        rid integer;
BEGIN
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    SELECT INTO did id from ka_dokumente_tbl WHERE name = filename AND mid = master_id;
    IF NOT FOUND THEN
        INSERT INTO ka_dokumente_tbl(master_id) VALUES (mid);
        SELECT INTO rid currval('ka_dokumente_tbl_id_seq');
    ELSE
        RETURN did;
    END IF;

    IF nuuid IS NOT NULL THEN
        UPDATE ka_dokumente_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('ka_dokumente_tbl', rid);
    END IF;
    RETURN rid;
END;
$$ ;

REVOKE ALL ON FUNCTION create_fall_dokument_ds(integer, char(36), varchar) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_fall_dokument_ds( integer, char(36), varchar) TO GROUP :cm_group;

CREATE OR REPLACE FUNCTION delete_fall_dokument_ds(did integer) RETURNS void
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid master_id FROM ka_dokumente_tbl WHERE id = did;
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;
    DELETE FROM ka_dokumente_tbl WHERE id = did;
END;
$$ ;

REVOKE ALL ON FUNCTION delete_fall_dokument_ds(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_fall_dokument_ds(integer)
                TO GROUP :cm_group;


-- Funktion, die alle Anhaenge zu einem gegebenen Fall loescht
CREATE OR REPLACE FUNCTION delete_all_fall_dokumente(mid integer)
RETURNS void
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
AS $$
BEGIN
    DELETE FROM ka_dokumente_tbl WHERE master_id = mid;
END;
$$;

REVOKE ALL     ON FUNCTION delete_all_fall_dokumente(integer) FROM PUBLIC;
GRANT  EXECUTE ON FUNCTION delete_all_fall_dokumente(integer) TO GROUP :admin_group;

-- Funktion zum Anlegen und Löschen von globalen Dokumenten
CREATE OR REPLACE FUNCTION create_global_dokument_ds(filename varchar) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    did integer;
BEGIN
    SELECT INTO did id from ka_dokumente_tbl WHERE name = filename and master_id IS NULL;
    IF  NOT FOUND THEN
        INSERT INTO ka_dokumente_tbl (master_id) VALUES (NULL);
        RETURN currval('ka_dokumente_tbl_id_seq');
    ELSE
        RETURN did;
    END IF;
END;
$$ ;

REVOKE ALL ON FUNCTION create_global_dokument_ds(varchar) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_global_dokument_ds(varchar) TO GROUP :admin_group;


CREATE OR REPLACE FUNCTION delete_global_dokument_ds(did integer) RETURNS void
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    DELETE FROM ka_dokumente_tbl WHERE id = did;
END;
$$ ;

REVOKE ALL ON FUNCTION delete_global_dokument_ds(did integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_global_dokument_ds( integer )
            TO GROUP :admin_group;

-- Trigger/Logging
CREATE TRIGGER t_ka_dokumente_tbl_insert AFTER
    INSERT ON ka_dokumente_tbl
    FOR EACH ROW EXECUTE PROCEDURE mpuls_log();

CREATE TRIGGER t_ka_dokumente_tbl_update AFTER
    UPDATE ON ka_dokumente_tbl
    FOR EACH ROW EXECUTE PROCEDURE mpuls_log();

CREATE TRIGGER t_ka_dokumente_tbl_delete AFTER
    DELETE ON ka_dokumente_tbl
    FOR EACH ROW EXECUTE PROCEDURE mpuls_log();

CREATE TRIGGER t1_ka_dokumente_tbl_insert AFTER
    INSERT ON ka_dokumente_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_ka_dokumente_tbl_update AFTER
    UPDATE ON ka_dokumente_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_ka_dokumente_tbl_delete AFTER
    DELETE ON ka_dokumente_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

COMMIT;
-- Create Views.
\i sql/install/views/06_documents.sql
