#! /usr/bin/env python
# -*- coding: utf-8 -*-

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import csv
from optparse import OptionParser

import os
import psycopg2

from datetime import datetime

#
# Configuration
#
query_list = [
    {'query': '''select * from ka_konfiguration_tbl WHERE bez = 'max_speicherdauer' LIMIT 1''',
     'name': '01-Akten'},

]

traeger_mapper = {'awo': {'port': 5432,
                          'case_nr': 3},
                  'ejsa': {'port': 5433,
                           'case_nr': 2},
                  'ib': {'port': 5434,
                         'case_nr': 4},
                  'kjs': {'port': 5435,
                          'case_nr': 1}
                  }
os.chdir('/tmp')


#
# Parse user options
#
parser = OptionParser()
parser.add_option("--verbose",
                  action="store_true",
                  dest="verbose",
                  default=False,
                  help="Print debug and other information statements")

(options, args) = parser.parse_args()

if options.verbose:
    print "\n** Start %s" % datetime.now()

#
# Process each Traeger
#
out = []
is_header = True
for traeger in traeger_mapper:
    show_query = True

    if options.verbose:
        print "**** Processing: {0}".format(traeger)

    port = traeger_mapper[traeger]['port']
    connection_global = psycopg2.connect(port=port)
    cur_global = connection_global.cursor()
    cur_global.execute("SELECT datname FROM pg_database WHERE datname SIMILAR TO 'ka\_[0-9]%' ORDER BY datname;")


    # Process each database:
    for database in cur_global:
        database_id = database[0]

        if options.verbose:
            print("  * database {0} (port: {1})".format(database_id, port))

        connection_live = psycopg2.connect(database=database_id, port=port)
        cur_live = connection_live.cursor()

        # Process all queries and write the results to disk
        for query in query_list:
            rows = []

            try:
                cur_live.execute(query['query'])
            except psycopg2.ProgrammingError as e:
                print ("ProgrammingError while executing query."
                       " Details follow: {0}".format(e))
                continue

            except psycopg2.InternalError as e:
                print ("InternalError while executing query."
                       " Details follow: {0}".format(e))
                continue


            # Append header row
            if is_header:
                out.append(['Traeger', 'Database', 'Query'] + [i[0] for i in cur_live.description])
                is_header = False

            # Append evaluation results
            for result in cur_live:
                out.append([traeger, database_id, query['name']] + [i for i in result])

        cur_live.close()

    cur_global.close()

with open('custom-eval.csv', 'wb') as f:
    writer = csv.writer(f)
    writer.writerows(out)

if options.verbose:
    print "** End %s" % datetime.now()


